# Generated by Django 2.2.28 on 2024-02-07 05:30

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django_paypal', '0004_paypalpayment_related_resource_state'),
    ]

    operations = [
        migrations.CreateModel(
            name='PaypalAPIPostData',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('url', models.CharField(blank=True, max_length=255, verbose_name='URL')),
                ('post_data', django.contrib.postgres.fields.jsonb.JSONField(verbose_name='Post Data')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
            ],
        ),
        migrations.CreateModel(
            name='PaypalOrder',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('order_id', models.CharField(max_length=255, unique=True, verbose_name='Order ID')),
                ('status', models.CharField(blank=True, max_length=255, verbose_name='Status')),
            ],
        ),
        migrations.CreateModel(
            name='PaypalWebhook',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('webhook_id', models.CharField(max_length=255, unique=True, verbose_name='Webhook ID')),
                ('url', models.CharField(max_length=255, unique=True, verbose_name='URL')),
                ('event_types', django.contrib.postgres.fields.jsonb.JSONField(verbose_name='Active Webhook Events')),
            ],
        ),
        migrations.RenameModel(
            old_name='PaypalItem',
            new_name='LegacyPaypalItem',
        ),
        migrations.RenameModel(
            old_name='PaypalPayment',
            new_name='LegacyPaypalPayment',
        ),
        migrations.RenameModel(
            old_name='PaypalTransaction',
            new_name='LegacyPaypalTransaction',
        ),
        migrations.AlterModelOptions(
            name='legacypaypalitem',
            options={'verbose_name': 'Legacy Paypal Item (payments v1 API)', 'verbose_name_plural': 'Legacy Paypal Item (payments v1 API)'},
        ),
        migrations.AlterModelOptions(
            name='legacypaypalpayment',
            options={'verbose_name': 'Legacy Paypal Payment (payments v1 API)', 'verbose_name_plural': 'Legacy Paypal Payments (payments v1 API)'},
        ),
        migrations.AlterModelOptions(
            name='legacypaypaltransaction',
            options={'verbose_name': 'Legacy Paypal Transaction (payments v1 API)', 'verbose_name_plural': 'Legacy Paypal Transactions (payments v1 API)'},
        ),
        migrations.CreateModel(
            name='PaypalWebhookEvent',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('payload', django.contrib.postgres.fields.jsonb.JSONField(verbose_name='Payload')),
                ('order', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='webhook_events', to='django_paypal.PaypalOrder')),
                ('webhook', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='events', to='django_paypal.PaypalWebhook')),
            ],
        ),
        migrations.CreateModel(
            name='PaypalAPIResponse',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('url', models.CharField(blank=True, max_length=255, verbose_name='URL')),
                ('response_data', django.contrib.postgres.fields.jsonb.JSONField(verbose_name='Response Data')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('order', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='api_responses', to='django_paypal.PaypalOrder')),
                ('post', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='responses', to='django_paypal.PaypalAPIPostData')),
            ],
        ),
        migrations.AddField(
            model_name='paypalapipostdata',
            name='order',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='api_posts', to='django_paypal.PaypalOrder'),
        ),
    ]



from dataclasses import dataclass
from typing import Optional, List, Literal, NamedTuple, Dict, Any
from dataclass_wizard import JSONWizard

ItemCategories = Literal['DIGITAL_GOODS', 'PHYSICAL_GOODS', 'DONATION']
ShippingType = Literal['SHIPPING', 'PICKUP_IN_PERSON', 'PICKUP_IN_STORE', 'PICKUP_FROM_PERSON']
CardType = Literal[
    'VISA',
    'MASTERCARD',
    'DISCOVER',
    'AMEX',
    'SOLO',
    'JCB',
    'STAR',
    'DELTA',
    'SWITCH',
    'MAESTRO',
    'CB_NATIONALE',
    'CONFIGOGA',
    'CONFIDIS',
    'ELECTRON',
    'CETELEM',
    'CHINA_UNION_PAY',
]
Intent = Literal['CAPTURE', 'AUTHORIZE']
ProcessingInstruction = Literal['ORDER_COMPLETE_ON_PAYMENT_APPROVAL', 'NO_INSTRUCTION']
PhoneType = Literal['FAX', 'HOME', 'MOBILE', 'OTHER', 'PAGER']


@dataclass
class OAuthResponse:
    scope: str
    access_token: str
    token_type: str
    app_id: str
    expires_in: int
    nonce: str


@dataclass
class Link:
    href: str
    rel: str
    method: Optional[Literal['GET', 'POST', 'PATCH', 'DELETE']] = None


@dataclass
class PhoneNumber:
    national_number: str
    extension_number: Optional[str] = None


@dataclass
class PhoneWithType:
    phone_number: PhoneNumber
    phone_type: Optional[PhoneType] = None


@dataclass
class Name:
    prefix: Optional[str] = None
    given_name: Optional[str] = None
    surname: Optional[str] = None
    middle_name: Optional[str] = None
    suffix: Optional[str] = None
    full_name: Optional[str] = None

    def __post_init__(self):
        if self.given_name and len(self.given_name) > 140:
            raise ValueError('Given name must be 140 characters or fewer')
        if self.surname and len(self.surname) > 140:
            raise ValueError('Surname must be 140 characters or fewer')


@dataclass
class ExperienceContext:
    brand_name: Optional[str] = None
    shipping_preference: Optional[Literal['GET_FROM_FILE', 'NO_SHIPPING', 'SET_PROVIDED_ADDRESS']] = None
    landing_page: Optional[Literal['LOGIN', 'GUEST_CHECKOUT', 'NO_PREFERENCE']] = None
    user_action: Optional[Literal['CONTINUE', 'PAY_NOW']] = None
    payment_method_preference: Optional[Literal['UNRESTRICTED', 'IMMEDIATE_PAYMENT_REQUIRED']] = None
    locale: Optional[str] = None
    return_url: Optional[str] = None
    cancel_url: Optional[str] = None


@dataclass
class TaxInfo:
    tax_id: Optional[str] = None
    tax_id_type: Optional[Literal['BR_CPF', 'BR_CNPJ']] = None


@dataclass
class AddressDetails:
    street_number: Optional[str] = None
    street_name: Optional[str] = None
    street_type: Optional[str] = None
    delivery_service: Optional[str] = None
    building_name: Optional[str] = None
    sub_building: Optional[str] = None


@dataclass
class Address:
    country_code: str
    address_line_1: Optional[str] = None
    address_line_2: Optional[str] = None
    address_line_3: Optional[str] = None
    admin_area_1: Optional[str] = None
    admin_area_2: Optional[str] = None
    admin_area_3: Optional[str] = None
    admin_area_4: Optional[str] = None
    postal_code: Optional[str] = None
    address_details: Optional[AddressDetails] = None


@dataclass
class Customer:
    id: Optional[str] = None
    email_address: Optional[str] = None
    merchant_customer_id: Optional[str] = None


@dataclass
class Vault:
    usage_type: str
    store_in_vault: Optional[str] = None
    description: Optional[str] = None
    owner_id: Optional[str] = None
    customer_type: Optional[str] = None
    permit_multiple_payment_tokens: Optional[bool] = None


@dataclass
class PayPalWalletAttributes:
    customer: Optional[Customer] = None
    vault: Optional[Vault] = None


@dataclass
class PayPalWallet:
    name: Optional[Name] = None
    phone: Optional[PhoneType] = None
    vault_id: Optional[str] = None
    email_address: Optional[str] = None
    birth_date: Optional[str] = None
    tax_info: Optional[TaxInfo] = None
    address: Optional[Address] = None
    attributes: Optional[PayPalWalletAttributes] = None
    experience_context: Optional[ExperienceContext] = None
    billing_agreement_id: Optional[str] = None


@dataclass
class PayPalWalletResponse:
    email_address: Optional[str] = None
    account_id: Optional[str] = None
    name: Optional[Name] = None
    phone_type: Optional[PhoneType] = None
    phone_number: Optional[PhoneNumber] = None
    birth_date: Optional[str] = None
    tax_info: Optional[TaxInfo] = None
    address: Optional[Address] = None
    attributes: Optional[PayPalWalletAttributes] = None


@dataclass
class PaymentSource(JSONWizard):
    class _(JSONWizard.Meta):
        key_transform_with_dump = 'SNAKE'

    paypal: PayPalWallet


@dataclass
class Amount:
    currency_code: str
    value: str


@dataclass
class Tax:
    currency_code: str
    value: str


@dataclass
class PurchaseItem:
    name: str
    quantity: int
    unit_amount: Amount
    description: Optional[str] = None
    sku: Optional[str] = None
    tax: Optional[Tax] = None
    category: Optional[ItemCategories] = None


@dataclass
class Breakdown:
    item_total: Optional[Amount] = None
    shipping: Optional[Amount] = None
    handling: Optional[Amount] = None
    tax_total: Optional[Amount] = None
    shipping_discount: Optional[Amount] = None
    discount: Optional[Amount] = None


@dataclass
class PurchaseUnitAmount(Amount):
    breakdown: Optional[Breakdown] = None


@dataclass
class Payee:
    email_address: Optional[str] = None
    merchant_id: Optional[str] = None


@dataclass
class PlatformFees:
    amount: Optional[Amount]
    payee_pricing_tier_id: Optional[str] = None
    payee_receivable_fx_rate_id: Optional[str] = None
    disbursement_mode: Optional[Literal['INSTANT', 'DELAYED']] = None


@dataclass
class PaymentInstruction:
    platform_fees: Optional[PlatformFees] = None


@dataclass
class ShippingOption:
    id: str
    label: str
    selected: bool
    type: Optional[ShippingType] = None
    amount: Optional[Amount] = None


@dataclass
class ShippingDetail:
    type: Optional[ShippingType] = None
    options: Optional[List[ShippingOption]] = None
    name: Optional[Name] = None
    address: Optional[Address] = None


@dataclass
class TrackerItem:
    name: Optional[str] = None
    quantity: Optional[str] = None
    sku: Optional[str] = None
    image_url: Optional[str] = None
    upc: Optional[Any] = None


@dataclass
class Tracker:
    id: Optional[str] = None
    status: Optional[Any] = None
    items: Optional[List[TrackerItem]] = None
    links: Optional[List[Link]] = None


@dataclass
class ShippingWithTrackingDetail(ShippingDetail):
    trackers: Optional[List[Tracker]] = None


@dataclass
class Level2Data:
    invoice_id: Optional[str] = None
    tax_total: Optional[Amount] = None


@dataclass
class LineItem:
    name: str
    quantity: int
    unit_amount: Amount
    description: Optional[str] = None
    sku: Optional[str] = None
    category: Optional[ItemCategories] = None
    tax: Optional[Amount] = None
    commodity_code: Optional[str] = None
    unit_of_measure: Optional[str] = None
    discount_amount: Optional[Amount] = None
    total_amount: Optional[Amount] = None


@dataclass
class Level3Data:
    ships_from_postal_code: Optional[str] = None
    line_items: Optional[List[LineItem]] = None
    shipping_amount: Optional[Amount] = None
    duty_amount: Optional[Amount] = None
    discount_amount: Optional[Amount] = None
    address: Optional[Address] = None


@dataclass
class SupplementaryCardData:
    level_2: Optional[Level2Data] = None
    level_3: Optional[Level3Data] = None


@dataclass
class SupplementaryData:
    card: SupplementaryCardData


@dataclass
class PurchaseUnit(JSONWizard):
    class _(JSONWizard.Meta):
        key_transform_with_dump = 'SNAKE'

    amount: Optional[PurchaseUnitAmount] = None
    reference_id: Optional[str] = None
    description: Optional[str] = None
    custom_id: Optional[str] = None
    invoice_id: Optional[str] = None
    soft_descriptor: Optional[str] = None
    items: Optional[List[PurchaseItem]] = None
    payee: Optional[Payee] = None
    payment_instruction: Optional[PaymentInstruction] = None
    shipping: Optional[ShippingWithTrackingDetail] = None
    supplementary_data: Optional[SupplementaryData] = None


@dataclass
class PreviousNetworkTransactionReference:
    id: str
    network: CardType
    date: Optional[str] = None
    acquirer_reference_number: Optional[str] = None


@dataclass
class StoredPaymentSource:
    payment_initiator: Literal['CUSTOMER', 'MERCHANT']
    payment_type: Literal['ONE_TIME', 'RECURRING', 'UNSCHEDULED']
    usage: Optional[Literal['FIRST', 'SUBSEQUENT', 'DERIVED']] = None
    previous_network_transaction_reference: Optional[PreviousNetworkTransactionReference] = None


@dataclass
class ApplicationContext(JSONWizard):
    class _(JSONWizard.Meta):
        key_transform_with_dump = 'SNAKE'

    stored_payment_source: StoredPaymentSource


class APIAuthCredentials(NamedTuple):
    client_id: str
    client_secret: str


@dataclass
class OrderCreatedAPIResponse(JSONWizard):
    class _(JSONWizard.Meta):
        key_transform_with_dump = 'SNAKE'

    id: str
    links: List[Link]
    payment_source: PaymentSource
    status: str


@dataclass
class Payer:
    payer_id: str
    address: Optional[Address] = None
    email_address: Optional[str] = None
    name: Optional[Name] = None


@dataclass
class PaymentSourceResponse:
    paypal: PayPalWalletResponse


@dataclass
class OrderDetailAPIResponse(JSONWizard):
    class _(JSONWizard.Meta):
        key_transform_with_dump = 'SNAKE'

    id: Optional[str] = None
    links: Optional[List[Link]] = None
    payer: Optional[Payer] = None
    payment_source: Optional[PaymentSourceResponse] = None
    purchase_units: Optional[List[PurchaseUnit]] = None
    status: Optional[str] = None
    create_time: Optional[str] = None
    update_time: Optional[str] = None
    processing_instruction: Optional[ProcessingInstruction] = None
    intent: Optional[Intent] = None


@dataclass
class SellerReceivableBreakdown:
    gross_amount: Amount
    net_amount: Optional[Amount] = None
    paypal_fee: Optional[Amount] = None


@dataclass
class SellerProtection:
    dispute_categories: Optional[List[str]] = None
    status: Optional[str] = None


@dataclass
class Capture:
    amount: Optional[Amount] = None
    id: Optional[str] = None
    links: Optional[List[Link]] = None
    create_time: Optional[str] = None
    final_capture: Optional[bool] = None
    seller_protection: Optional[SellerProtection] = None
    seller_receivable_breakdown: Optional[SellerReceivableBreakdown] = None
    status: Optional[str] = None
    update_time: Optional[str] = None


@dataclass
class Payments:
    captures: List[Capture]


@dataclass
class OrderCaptureAPIResponse(OrderDetailAPIResponse):
    pass


class WebhookEvent(NamedTuple):
    create_time: str
    event_type: str
    event_version: str
    id: str
    links: List[Link]
    resource: Dict[str, Any]
    resource_type: str
    resource_version: str
    summary: str



# Generated by Django 2.2.28 on 2024-04-24 05:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_paypal', '0005_auto_20240207_0630'),
    ]

    operations = [
        migrations.AddField(
            model_name='paypalwebhook',
            name='auth_hash',
            field=models.CharField(default='', max_length=255, verbose_name='Auth Hash'),
        ),
        migrations.AlterField(
            model_name='paypalwebhook',
            name='url',
            field=models.CharField(max_length=255, verbose_name='URL'),
        ),
        migrations.AlterUniqueTogether(
            name='paypalwebhook',
            unique_together={('url', 'auth_hash')},
        ),
    ]



from django_paypal import settings as django_paypal_settings


def build_paypal_full_uri(url):
    if url.startswith('/'):
        url = '{0}{1}'.format(django_paypal_settings.PAYPAL_ROOT_URL, url)
    return url



from django.conf import settings

from django_paypal.__init__ import __version__

DJANGO_PAYPAL_VERSION = __version__

PAYPAL_API_CLIENT_ID = getattr(settings, 'PAYPAL_API_CLIENT_ID', False)
PAYPAL_API_SECRET = getattr(settings, 'PAYPAL_API_SECRET', False)

PAYPAL_API_URL = getattr(settings, 'PAYPAL_API_URL', 'https://api-m.paypal.com')
PAYPAL_SANDBOX_API_URL = getattr(settings, 'PAYPAL_SANDBOX_API_URL', 'https://api-m.sandbox.paypal.com')
PAYPAL_SANDBOX = getattr(settings, 'PAYPAL_SANDBOX', True)

PAYPAL_AUTH_URL = getattr(settings, 'PAYPAL_AUTH_URL', '/v1/oauth2/token')
PAYPAL_AUTH_CACHE_TIMEOUT = getattr(settings, 'PAYPAL_AUTH_CACHE_TIMEOUT', 600)  # 10 minutes
PAYPAL_AUTH_CACHE_KEY = getattr(settings, 'PAYPAL_AUTH_CACHE_KEY', 'django-paypal-auth-{auth_hash}')
PAYPAL_ORDERS_API_ENDPOINT = getattr(settings, 'PAYPAL_ORDERS_API_ENDPOINT', '/v2/checkout/orders')
PAYPAL_WEBHOOK_LISTENER = getattr(settings, 'PAYPAL_WEBHOOK_LISTENER', None)

# checkout urls
PAYPAL_SUCCESS_URL = getattr(settings, 'PAYPAL_SUCCESS_URL', '/')
PAYPAL_CANCELLATION_URL = getattr(settings, 'PAYDIREKT_CANCELLATION_URL', '/')
PAYPAL_NOTIFICATION_URL = getattr(settings, 'PAYDIREKT_NOTIFICATION_URL', '/paypal/notify/')

if getattr(settings, 'PAYPAL', False):
    PAYPAL_ROOT_URL = settings.PAYPAL_ROOT_URL



# -*- coding: utf-8 -*-
# Generated by Django 1.11.27 on 2020-01-24 14:28
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django_paypal', '0002_auto_20171002_1551'),
    ]

    operations = [
        migrations.AddField(
            model_name='paypalpayment',
            name='initial_response_object',
            field=models.TextField(blank=True, null=True, verbose_name='initial post response'),
        ),
        migrations.AddField(
            model_name='paypalpayment',
            name='related_resource_id',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='related resource id'),
        ),
        migrations.AddField(
            model_name='paypalpayment',
            name='update_response_object',
            field=models.TextField(blank=True, null=True, verbose_name='updated get response'),
        ),
        migrations.AlterField(
            model_name='paypalitem',
            name='transaction',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='items', to='django_paypal.PaypalTransaction', verbose_name='transaction'),
        ),
        migrations.AlterField(
            model_name='paypaltransaction',
            name='payment',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='transactions', to='django_paypal.PaypalPayment', verbose_name='payment'),
        ),
    ]



# Generated by Django 2.2.28 on 2022-05-03 10:27

from django.db import migrations, models


def migrate_resource_state(apps, schema_editor):
    import json
    PaypalPayment = apps.get_model('django_paypal', 'PaypalPayment')

    for payment in PaypalPayment.objects.all():
        if payment.update_response_object:
            payment_response = json.loads(payment.update_response_object)
            if payment_response['state'] == 'approved':
                for transaction in payment_response['transactions']:
                    if 'related_resources' in transaction:
                        for resource in transaction['related_resources']:
                            if 'sale' in resource:
                                if 'state' in resource['sale']:
                                    payment.related_resource_state = resource['sale']['state']
                                    payment.save(update_fields=['related_resource_state'])


class Migration(migrations.Migration):

    dependencies = [
        ('django_paypal', '0003_auto_20200124_1528'),
    ]

    operations = [
        migrations.AddField(
            model_name='paypalpayment',
            name='related_resource_state',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='related resource state'),
        ),
        migrations.RunPython(migrate_resource_state, reverse_code=migrations.RunPython.noop, elidable=True)
    ]



from __future__ import unicode_literals


from decimal import Decimal
from typing import List

from dataclass_wizard import fromdict
from django.db import models
from django.utils.translation import gettext_lazy as _

from django_paypal.api_types import Capture

try:
    # Django 3.1 and newer
    from django.db.models import JSONField
except ImportError:
    # Django 2.2
    from django.contrib.postgres.fields import JSONField


class PaypalOrder(models.Model):
    order_id = models.CharField(_('Order ID'), max_length=255, unique=True)
    status = models.CharField(_('Status'), max_length=255, blank=True)
    capture_id = models.JSONField(_('Capture ID'), null=True, default=list)

    def get_capture_api_responses(self) -> models.QuerySet:
        return self.api_responses.filter(url__contains=f'/v2/checkout/orders/{self.order_id}/capture')

    @property
    def captures(self) -> List[Capture]:
        captures: List[Capture] = list()
        for capture_response in self.get_capture_api_responses():
            for purchase_unit in capture_response.response_data.get('purchase_units', []):
                for capture in purchase_unit.get('payments', {}).get('captures', []):
                    captures.append(fromdict(Capture, capture))
        return captures


class PaypalAPIPostData(models.Model):
    order = models.ForeignKey(PaypalOrder, related_name='api_posts', on_delete=models.CASCADE)
    url = models.CharField(_('URL'), max_length=255, blank=True)
    post_data = JSONField(_('Post Data'))
    created_at = models.DateTimeField(auto_now_add=True)


class PaypalAPIResponse(models.Model):
    order = models.ForeignKey(PaypalOrder, related_name='api_responses', on_delete=models.CASCADE)
    post = models.ForeignKey(PaypalAPIPostData, related_name='responses', null=True, on_delete=models.CASCADE)
    url = models.CharField(_('URL'), max_length=255, blank=True)
    response_data = JSONField(_('Response Data'))
    created_at = models.DateTimeField(auto_now_add=True)


class PaypalWebhook(models.Model):
    webhook_id = models.CharField(_('Webhook ID'), max_length=255, unique=True)
    auth_hash = models.CharField(_('Auth Hash'), max_length=255, default='')
    url = models.CharField('URL', max_length=255)
    event_types = JSONField(_('Active Webhook Events'))

    class Meta:
        unique_together = (
            'url',
            'auth_hash',
        )


class PaypalWebhookEvent(models.Model):
    payload = JSONField(_('Payload'))
    order = models.ForeignKey(PaypalOrder, related_name='webhook_events', on_delete=models.CASCADE)
    webhook = models.ForeignKey(PaypalWebhook, related_name='events', on_delete=models.CASCADE)


# The following models solely exist to keep payments made in version <0.3.0 stored in the database.
# They are not used in the current version of the app and have been created using /v1/payments.
# They might get removed in a future version.
class LegacyPaypalPayment(models.Model):
    payment_id = models.CharField(_('payment id'), max_length=255, unique=True)  # id created by paypal
    intent = models.CharField(_('intent'), max_length=255)  # sale
    experience_profile_id = models.CharField(_('intent'), max_length=255, blank=True)  # sale
    note_to_payer = models.CharField(_('intent'), max_length=165, blank=True)  # sale
    state = models.CharField(_('state'), max_length=255, blank=True)
    payment_method = models.CharField(_('payment method'), max_length=255, blank=True)
    custom = models.CharField(_('payment method'), max_length=255, blank=True)
    payer_id = models.CharField(_('payer id'), max_length=255, blank=True)
    transaction_fee = models.DecimalField(_('transaction fee'), max_digits=9, decimal_places=2, default=Decimal(0))
    related_resource_id = models.CharField(_('related resource id'), max_length=255, blank=True, null=True)
    related_resource_state = models.CharField(_('related resource state'), max_length=255, blank=True, null=True)
    initial_response_object = models.TextField(_('initial post response'), null=True, blank=True)
    update_response_object = models.TextField(_('updated get response'), null=True, blank=True)

    failure_reason = models.CharField(_('failure reason'), max_length=255, blank=True)

    link = models.URLField(_('link'), blank=True)
    approve_link = models.URLField(_('approve link'), blank=True)
    execute_link = models.URLField(_('execute link'), blank=True)

    created_at = models.DateTimeField(_('created at'), auto_now_add=True)
    last_modified = models.DateTimeField(_('last modified'), auto_now=True)

    objects = models.Manager()

    def __str__(self):
        return self.payment_id

    class Meta:
        verbose_name = _('Legacy Paypal Payment (payments v1 API)')
        verbose_name_plural = _('Legacy Paypal Payments (payments v1 API)')


class LegacyPaypalTransaction(models.Model):
    payment = models.ForeignKey(LegacyPaypalPayment, verbose_name=_('payment'), related_name='transactions', on_delete=models.PROTECT)

    total_amount = models.DecimalField(_('total amount'), max_digits=9, decimal_places=2)
    currency = models.CharField(_('currency'), max_length=10)
    subtotal = models.DecimalField(_('subtotal'), max_digits=9, decimal_places=2)
    tax = models.DecimalField(_('tax'), max_digits=9, decimal_places=2, blank=True)
    shipping = models.DecimalField(_('shipping'), max_digits=9, decimal_places=2, blank=True)
    handling_fee = models.DecimalField(_('handling fee'), max_digits=9, decimal_places=2, blank=True)
    shipping_discount = models.DecimalField(_('shipping discount'), max_digits=9, decimal_places=2, blank=True)
    insurance = models.DecimalField(_('insurance fee'), max_digits=9, decimal_places=2, blank=True)
    gift_wrap = models.DecimalField(_('gift wrap fee'), max_digits=9, decimal_places=2, blank=True)

    reference_id = models.CharField(_('reference id'), max_length=255, blank=True)
    settlement_destination = models.CharField(_('settlement destination'), max_length=255, default='PARTNER_BALANCE')
    allowed_payment_method = models.CharField(
        _('allowed payment method'), max_length=255, default='INSTANT_FUNDING_SOURCE'
    )  # UNRESTRICTED or INSTANT_FUNDING_SOURCE or IMMEDIATE_PAY
    description = models.CharField(_('description'), max_length=255, blank=True)
    note_to_payee = models.CharField(_('note to payee'), max_length=255, blank=True)
    custom = models.CharField(_('custom'), max_length=127, blank=True)
    invoice_number = models.CharField(_('invoice number'), max_length=127)
    purchase_order = models.CharField(_('purchase order'), max_length=127)
    soft_descriptor = models.CharField(_('soft descriptor'), max_length=22)

    created_at = models.DateTimeField(_('created at'), auto_now_add=True)
    last_modified = models.DateTimeField(_('last modified'), auto_now=True)

    shipping_method = models.CharField(_('shipping method'), max_length=255, blank=True)
    shipping_phone_number = models.CharField(_('shipping phone number'), max_length=255, blank=True)

    shipping_address_line_1 = models.CharField(_('shipping address line 1'), max_length=100)  # number, street, etc.
    shipping_address_line_2 = models.CharField(_('shipping address line 2'), max_length=100, blank=True)  # apt number, etc.
    shipping_address_city = models.CharField(_('shipping address city'), max_length=50)
    shipping_address_country = models.CharField(_('shipping address country'), max_length=2)
    shipping_address_postal_code = models.CharField(
        _('shipping address postal code'), max_length=20, blank=True
    )  # required in certain countries
    shipping_address_state = models.CharField(_('shipping address state'), max_length=100, blank=True)
    shipping_address_phone = models.CharField(_('shipping address phone'), max_length=50)
    shipping_address_normalization_status = models.CharField(
        _('shipping address normalization status'), max_length=50, default='UNKNOWN'
    )  # UNKNOWN or UNNORMALIZED_USER_PREFERRED or NORMALIZED or UNNORMALIZED
    shipping_address_type = models.CharField(
        _('shipping address type'), max_length=50, default='HOME_OR_WORK'
    )  # HOME_OR_WORK or GIFT or other
    shipping_address_recipient_name = models.CharField(_('shipping address recipient name'), max_length=127)

    objects = models.Manager()

    def __str__(self):
        return '{} - {} {}'.format(self.reference_id, self.total_amount, self.currency)

    class Meta:
        verbose_name = _('Legacy Paypal Transaction (payments v1 API)')
        verbose_name_plural = _('Legacy Paypal Transactions (payments v1 API)')


class LegacyPaypalItem(models.Model):
    transaction = models.ForeignKey(LegacyPaypalTransaction, verbose_name=_('transaction'), related_name='items', on_delete=models.PROTECT)

    sku = models.CharField(_('stock keeping unit'), max_length=127)
    name = models.CharField(_('name'), max_length=127)
    description = models.CharField(_('description'), max_length=127, blank=True)
    quantity = models.CharField(_('quantity'), max_length=10)
    price = models.CharField(_('price'), max_length=10)
    currency = models.CharField(_('currency'), max_length=3)
    tax = models.CharField(_('currency'), max_length=10, blank=True)
    url = models.URLField(_('url'), blank=True)

    created_at = models.DateTimeField(_('created at'), auto_now_add=True)
    last_modified = models.DateTimeField(_('last modified'), auto_now=True)

    objects = models.Manager()

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _('Legacy Paypal Item (payments v1 API)')
        verbose_name_plural = _('Legacy Paypal Item (payments v1 API)')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from decimal import Decimal
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_paypal', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='paypalpayment',
            name='approve_link',
            field=models.URLField(verbose_name='approve link', blank=True),
        ),
        migrations.AlterField(
            model_name='paypalpayment',
            name='custom',
            field=models.CharField(max_length=255, verbose_name='payment method', blank=True),
        ),
        migrations.AlterField(
            model_name='paypalpayment',
            name='link',
            field=models.URLField(verbose_name='link', blank=True),
        ),
        migrations.AlterField(
            model_name='paypalpayment',
            name='payer_id',
            field=models.CharField(max_length=255, verbose_name='payer id', blank=True),
        ),
        migrations.AlterField(
            model_name='paypalpayment',
            name='payment_method',
            field=models.CharField(max_length=255, verbose_name='payment method', blank=True),
        ),
        migrations.AddField(
            model_name='paypalpayment',
            name='transaction_fee',
            field=models.DecimalField(default=Decimal('0'), verbose_name='transaction fee', max_digits=9, decimal_places=2),
        ),
    ]



from __future__ import unicode_literals

import hashlib
import json
import warnings
from typing import Literal, List, Any, Dict

from django.core.cache import cache
import requests
from django.http import HttpRequest
from requests.auth import HTTPBasicAuth

from django_paypal import settings as django_paypal_settings
from django_paypal.exceptions import PaypalAuthFailure, PaypalAPIError, PaypalWebhookVerificationError, PaypalOrderAlreadyCapturedError
from django_paypal.models import (
    PaypalOrder,
    PaypalAPIPostData,
    PaypalAPIResponse,
    PaypalWebhook,
)
from django_paypal.api_types import (
    OAuthResponse,
    PaymentSource,
    ApplicationContext,
    ExperienceContext,
    PurchaseUnit,
    APIAuthCredentials,
    OrderCreatedAPIResponse,
    OrderCaptureAPIResponse,
    OrderDetailAPIResponse,
    Intent,
)
from django_paypal.utils import build_paypal_full_uri
from django_paypal.signals import order_captured, order_created


class PaypalWrapper(object):
    interface_version = 'django_paypal_v{}'.format(django_paypal_settings.DJANGO_PAYPAL_VERSION)

    api_url = django_paypal_settings.PAYPAL_API_URL
    sandbox_url = django_paypal_settings.PAYPAL_SANDBOX_API_URL
    orders_api_endpoint = django_paypal_settings.PAYPAL_ORDERS_API_ENDPOINT
    auth_url = django_paypal_settings.PAYPAL_AUTH_URL
    auth_cache_timeout = django_paypal_settings.PAYPAL_AUTH_CACHE_TIMEOUT
    auth_cache_key = django_paypal_settings.PAYPAL_AUTH_CACHE_KEY

    auth: APIAuthCredentials = None

    def __init__(self, auth: APIAuthCredentials, sandbox=None):
        super(PaypalWrapper, self).__init__()
        self.auth = auth
        if sandbox or django_paypal_settings.PAYPAL_SANDBOX:
            self.api_url = self.sandbox_url

    def create_order(
        self,
        intent: Intent,
        purchase_units: List[PurchaseUnit],
        payment_source: PaymentSource,
        application_context: ApplicationContext = None,
        success_url=django_paypal_settings.PAYPAL_SUCCESS_URL,
        cancellation_url=django_paypal_settings.PAYPAL_CANCELLATION_URL,
    ) -> OrderCreatedAPIResponse:
        if payment_source.paypal and (success_url or cancellation_url):
            if not payment_source.paypal.experience_context:
                payment_source.paypal.experience_context = ExperienceContext(
                    return_url=build_paypal_full_uri(success_url),
                    cancel_url=build_paypal_full_uri(cancellation_url),
                )
            else:
                if not payment_source.paypal.experience_context.return_url:
                    payment_source.paypal.experience_context.return_url = build_paypal_full_uri(success_url)
                if not payment_source.paypal.experience_context.cancel_url:
                    payment_source.paypal.experience_context.cancel_url = build_paypal_full_uri(cancellation_url)

        order_data = {'intent': intent, 'purchase_units': [purchase_unit.to_dict() for purchase_unit in purchase_units]}
        if payment_source:
            order_data.update({'payment_source': payment_source.to_dict()})
        if application_context:
            order_data.update({'application_context': application_context.to_dict()})

        url = '{0}{1}'.format(self.api_url, self.orders_api_endpoint)
        order_response_dict = self.call_api(url=url, data=order_data, method='POST')
        order_response = OrderCreatedAPIResponse.from_dict(order_response_dict)
        new_order = PaypalOrder.objects.create(order_id=order_response.id, status=order_response.status)
        post_obj = PaypalAPIPostData.objects.create(order=new_order, url=url, post_data=order_data)
        PaypalAPIResponse.objects.create(order=new_order, url=url, response_data=order_response_dict, post=post_obj)
        order_created.send(sender=self.__class__, order=new_order, response=order_response_dict)
        return order_response

    def capture_order(self, order_id: str) -> OrderCaptureAPIResponse:
        order = PaypalOrder.objects.get(order_id=order_id)
        url = f'{self.api_url}{self.orders_api_endpoint}/{order.order_id}/capture'

        try:
            order_capture_response = self.call_api(url=url, method='POST')
        except PaypalAPIError as e:
            if e.response.status_code == 422:
                order_capture_response = e.response.json()
                if order_capture_response.get('name') == 'UNPROCESSABLE_ENTITY':
                    if order_capture_response['details'][0]['issue'] == 'ORDER_ALREADY_CAPTURED':
                        raise PaypalOrderAlreadyCapturedError(str(e), response=e.response)
            raise e

        order_capture = OrderCaptureAPIResponse.from_dict(order_capture_response)
        captures_id_list = []
        for purchase_unit in order_capture.purchase_units:
            for capture in purchase_unit.get('payments', {}).get('captures', []):
                if capture.id and capture.reference_id == order_id:
                    if capture.id in order.capture_id:
                        raise PaypalOrderAlreadyCapturedError('Order already captured', response=order_capture_response)
                    captures_id_list.append(capture.id)

        order.status = order_capture.status
        order.capture_id = captures_id_list
        order.save(update_fields=['status', 'capture_id'])
        post_obj = PaypalAPIPostData.objects.create(order=order, url=url, post_data={})
        PaypalAPIResponse.objects.create(order=order, url=url, response_data=order_capture_response, post=post_obj)
        order_captured.send(sender=self.__class__, order=order, response=order_capture_response)
        return order_capture

    def get_order_details(self, order_id: str) -> OrderDetailAPIResponse:
        order = PaypalOrder.objects.get(order_id=order_id)
        url = f'{self.api_url}{self.orders_api_endpoint}/{order.order_id}'
        order_details_response = self.call_api(url=url, method='GET')
        return OrderDetailAPIResponse.from_dict(order_details_response)

    def call_api(self, url: str, method: Literal['GET', 'POST', 'PATCH', 'DELETE'], data=None) -> Dict[str, Any]:
        headers = {'Authorization': f'Bearer {self._get_access_token()}', 'Content-Type': 'application/json'}

        try:
            if method == 'GET':
                response = requests.get(url, headers=headers)
            elif method == 'POST':
                response = requests.post(url, headers=headers, json=data)
            elif method == 'PATCH':
                response = requests.patch(url, headers=headers, json=data)
            elif method == 'DELETE':
                response = requests.delete(url, headers=headers)
                response.raise_for_status()
                return {'deleted': True}
            else:
                raise ValueError('Invalid method')
            response.raise_for_status()
            response_json = response.json()
            return response_json
        except requests.HTTPError as e:
            raise PaypalAPIError(str(e), response=e.response)

    def setup_webhooks(self, webhook_listener: str) -> PaypalWebhook:
        if PaypalWebhook.objects.filter(url=webhook_listener, auth_hash=self.api_auth_hash).exists():
            raise ValueError(f'Webhook listener ({webhook_listener}) already exists')
        from django_paypal.webhooks import WebhookEvents

        data = {'url': webhook_listener, 'event_types': [{'name': event} for event in WebhookEvents.ORDERS]}
        webhook_api = '{0}{1}'.format(self.api_url, '/v1/notifications/webhooks')
        try:
            response_dict = self.call_api(url=webhook_api, method='POST', data=data)
            return PaypalWebhook.objects.create(
                webhook_id=response_dict['id'],
                auth_hash=self.api_auth_hash,
                url=response_dict['url'],
                event_types=response_dict['event_types'],
            )
        except PaypalAPIError as e:
            response_json = e.response.json()
            if response_json.get('name') == 'WEBHOOK_URL_ALREADY_EXISTS':
                webhook_list = self.call_api(url=webhook_api, method='GET')
                for webhook in webhook_list.get('webhooks', []):
                    if webhook['url'] == webhook_listener:
                        return PaypalWebhook.objects.create(
                            webhook_id=webhook['id'], url=webhook['url'], event_types=webhook['event_types'], auth_hash=self.api_auth_hash
                        )
            raise e

    def patch_webhook(self, webhook_id: str, patch_data: List[Dict]) -> PaypalWebhook:
        try:
            paypal_webhook = PaypalWebhook.objects.get(webhook_id=webhook_id)
        except PaypalWebhook.DoesNotExist as e:
            warnings.warn(f'Webhook with id {webhook_id} does not exist in the database. Set up webhooks by calling "setup_webhooks"')
            raise e
        url = '{0}{1}'.format(self.api_url, f'/v1/notifications/webhooks/{paypal_webhook.webhook_id}')
        webhook_response_dict = self.call_api(url=url, method='PATCH', data=patch_data)
        paypal_webhook.url = webhook_response_dict['url']
        paypal_webhook.event_types = webhook_response_dict['event_types']
        paypal_webhook.save()
        return paypal_webhook

    def delete_webhook(self, webhook_listener: str) -> bool:
        try:
            paypal_webhook = PaypalWebhook.objects.get(url=webhook_listener, auth_hash=self.api_auth_hash)
        except PaypalWebhook.DoesNotExist as e:
            raise e
        url = '{0}{1}'.format(self.api_url, f'/v1/notifications/webhooks/{paypal_webhook.webhook_id}')
        self.call_api(url=url, method='DELETE')
        paypal_webhook.delete()
        return True

    def verify_webhook_event(self, request: HttpRequest, webhook_id: str) -> bool:
        headers = request.headers
        auth_algo = headers.get('Paypal-Auth-Algo')
        cert_url = headers.get('Paypal-Cert-Url')
        transmission_id = headers.get('Paypal-Transmission-Id')
        transmission_time = headers.get('Paypal-Transmission-Time')
        transmission_sig = headers.get('Paypal-Transmission-Sig')
        request_data = json.loads(request.body.decode('utf-8'))
        verification_payload = {
            'auth_algo': auth_algo,
            'cert_url': cert_url,
            'transmission_id': transmission_id,
            'transmission_time': transmission_time,
            'transmission_sig': transmission_sig,
            'webhook_id': webhook_id,
            'webhook_event': request_data,
        }
        url = f'{self.api_url}/v1/notifications/verify-webhook-signature'
        res = self.call_api(url=url, method='POST', data=verification_payload)
        if res.get('verification_status') == 'SUCCESS':
            return True
        raise PaypalWebhookVerificationError('Webhook verification failed', res)

    def verify_api_keys(self) -> bool:
        self._get_access_token()
        return True

    def _get_access_token(self) -> str:
        if not self.auth_cache_timeout:
            auth_response = self._authorize_client()
            return auth_response.access_token
        else:
            cache_key = self.auth_cache_key.format(auth_hash=self.api_auth_hash)
            access_token = cache.get(cache_key)
            if not access_token:
                auth_response = self._authorize_client()
                access_token = auth_response.access_token
                cache.set(cache_key, access_token, self.auth_cache_timeout)
            return access_token

    def _authorize_client(self) -> OAuthResponse:
        # preparing request
        url = f'{self.api_url}{self.auth_url}'

        api_auth = HTTPBasicAuth(self.auth.client_id, self.auth.client_secret)
        headers = {'Accept': 'application/json', 'Content-Type': 'application/x-www-form-urlencoded'}
        data = {'grant_type': 'client_credentials'}

        try:
            response = requests.post(url, headers=headers, data=data, auth=api_auth)
            response.raise_for_status()  # Raise an exception for HTTP errors
            response_json = response.json()
            return OAuthResponse(**response_json)
        except requests.HTTPError as e:
            raise PaypalAuthFailure(str(e), response=e.response)

    @property
    def api_auth_hash(self) -> str:
        return hashlib.md5(f'{self.auth.client_id}{self.auth.client_secret}'.encode()).hexdigest()



import json
from typing import Any, Dict

from django.conf import settings
from django.http import HttpResponse, HttpRequest
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from django.views.generic import View

from .api_types import APIAuthCredentials
from .models import PaypalWebhook, PaypalWebhookEvent, PaypalOrder
from .signals import order_approved, order_completed
from .wrappers import PaypalWrapper
from django_paypal import settings as django_paypal_settings


class WebhookEvents:
    ORDERS = ['CHECKOUT.ORDER.COMPLETED', 'CHECKOUT.ORDER.APPROVED', 'CHECKOUT.PAYMENT-APPROVAL.REVERSED']


def verify_and_save_webhook_event(request: HttpRequest, paypal_wrapper: PaypalWrapper, payload: Dict[str, Any]):
    if not settings.DEBUG:
        paypal_webhook = PaypalWebhook.objects.get(url=request.build_absolute_uri(), auth_hash=paypal_wrapper.api_auth_hash)
        paypal_wrapper.verify_webhook_event(request, paypal_webhook.webhook_id)
        try:
            paypal_order = PaypalOrder.objects.get(order_id=payload['resource']['id'])
        except (KeyError, TypeError):  # if the payload does not contain the order id
            paypal_order = None
        event_data = {'payload': payload, 'webhook': paypal_webhook, 'order': paypal_order}
        PaypalWebhookEvent.objects.create(**event_data)


@method_decorator(csrf_exempt, name='dispatch')
class PaypalWebhookView(View):
    def post(self, request, *args, **kwargs):
        post_dict = json.loads(request.body.decode('utf-8'))
        event_type = post_dict.get('event_type')
        if event_type not in WebhookEvents.ORDERS:
            return HttpResponse(status=400)

        paypal_wrapper = PaypalWrapper(
            auth=APIAuthCredentials(
                client_id=django_paypal_settings.PAYPAL_API_CLIENT_ID, client_secret=django_paypal_settings.PAYPAL_API_SECRET
            )
        )

        verify_and_save_webhook_event(request, paypal_wrapper, payload=post_dict)

        if event_type == 'CHECKOUT.ORDER.APPROVED':
            order_approved.send(sender=self.__class__, resource=post_dict.get('resource'))
            paypal_wrapper.capture_order(post_dict.get('resource').get('id'))
        if event_type == 'CHECKOUT.ORDER.COMPLETED':
            order_completed.send(sender=self.__class__, resource=post_dict.get('resource'))

        return HttpResponse(status=200)



from django.db import migrations, models

class Migration(migrations.Migration):

    dependencies = [
        ('django_paypal', '0006_auto_20240424_0727'),
    ]

    operations = [
        migrations.AddField(
            model_name='PaypalOrder',
            name='capture_id',
            field=models.JSONField(verbose_name='Capture ID', null=True, default=list),
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup
import re
import os
import sys


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, "__init__.py")).read()
    return re.match("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)


def get_packages(package):
    """
    Return root package and all sub-packages.
    """
    return [
        dirpath
        for dirpath, dirnames, filenames in os.walk(package)
        if os.path.exists(os.path.join(dirpath, "__init__.py"))
    ]


def get_package_data(package):
    """
    Return all files under the root package, that are not in a
    package themselves.
    """
    walk = [
        (dirpath.replace(package + os.sep, "", 1), filenames)
        for dirpath, dirnames, filenames in os.walk(package)
        if "tests" not in dirnames
        and not os.path.exists(os.path.join(dirpath, "__init__.py"))
    ]

    filepaths = []
    for base, filenames in walk:
        filepaths.extend([os.path.join(base, filename) for filename in filenames])
    return {package: filepaths}


REQUIREMENTS = [
    "Django>=2.2",
    "certifi>=2018.10.15",
    "requests>=2.20.0",
    "dataclass-wizard>=0.2.2",
]

version = get_version("django_paypal")

if sys.argv[-1] == "publish":
    os.system("python setup.py sdist upload")
    print("You probably want to also tag the version now:")
    print("  git tag -a %s -m 'version %s'" % (version, version))
    print("  git push --tags")
    sys.exit()


setup(
    name="django-paypal-plus",
    author="Particulate Solutions GmbH",
    author_email="tech@particulate.me",
    description="Django integration of PayPal's Orders v2 API",
    version=version,
    url="https://github.com/ParticulateSolutions/django-paypal-plus",
    packages=get_packages("django_paypal"),
    package_data=get_package_data("django_paypal"),
    include_package_data=True,
    classifiers=[
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Framework :: Django",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    install_requires=REQUIREMENTS,
    zip_safe=False,
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='PaypalItem',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('sku', models.CharField(max_length=127, verbose_name='stock keeping unit')),
                ('name', models.CharField(max_length=127, verbose_name='name')),
                ('description', models.CharField(max_length=127, verbose_name='description', blank=True)),
                ('quantity', models.CharField(max_length=10, verbose_name='quantity')),
                ('price', models.CharField(max_length=10, verbose_name='price')),
                ('currency', models.CharField(max_length=3, verbose_name='currency')),
                ('tax', models.CharField(max_length=10, verbose_name='currency', blank=True)),
                ('url', models.URLField(verbose_name='url', blank=True)),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='created at')),
                ('last_modified', models.DateTimeField(auto_now=True, verbose_name='last modified')),
            ],
            options={
                'verbose_name': 'Paypal Item',
                'verbose_name_plural': 'Paypal Item',
            },
        ),
        migrations.CreateModel(
            name='PaypalPayment',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('payment_id', models.CharField(unique=True, max_length=255, verbose_name='payment id')),
                ('intent', models.CharField(max_length=255, verbose_name='intent')),
                ('experience_profile_id', models.CharField(max_length=255, verbose_name='intent', blank=True)),
                ('note_to_payer', models.CharField(max_length=165, verbose_name='intent', blank=True)),
                ('state', models.CharField(max_length=255, verbose_name='state', blank=True)),
                ('payment_method', models.CharField(max_length=255, verbose_name='payment method')),
                ('custom', models.CharField(max_length=255, verbose_name='payment method')),
                ('payer_id', models.CharField(max_length=255, verbose_name='payer id')),
                ('failure_reason', models.CharField(max_length=255, verbose_name='failure reason', blank=True)),
                ('link', models.URLField(verbose_name='link')),
                ('approve_link', models.URLField(verbose_name='approve link')),
                ('execute_link', models.URLField(verbose_name='execute link', blank=True)),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='created at')),
                ('last_modified', models.DateTimeField(auto_now=True, verbose_name='last modified')),
            ],
            options={
                'verbose_name': 'Paypal Payment',
                'verbose_name_plural': 'Paypal Payments',
            },
        ),
        migrations.CreateModel(
            name='PaypalTransaction',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('total_amount', models.DecimalField(verbose_name='total amount', max_digits=9, decimal_places=2)),
                ('currency', models.CharField(max_length=10, verbose_name='currency')),
                ('subtotal', models.DecimalField(verbose_name='subtotal', max_digits=9, decimal_places=2)),
                ('tax', models.DecimalField(verbose_name='tax', max_digits=9, decimal_places=2, blank=True)),
                ('shipping', models.DecimalField(verbose_name='shipping', max_digits=9, decimal_places=2, blank=True)),
                ('handling_fee', models.DecimalField(verbose_name='handling fee', max_digits=9, decimal_places=2, blank=True)),
                ('shipping_discount', models.DecimalField(verbose_name='shipping discount', max_digits=9, decimal_places=2, blank=True)),
                ('insurance', models.DecimalField(verbose_name='insurance fee', max_digits=9, decimal_places=2, blank=True)),
                ('gift_wrap', models.DecimalField(verbose_name='gift wrap fee', max_digits=9, decimal_places=2, blank=True)),
                ('reference_id', models.CharField(max_length=255, verbose_name='reference id', blank=True)),
                ('settlement_destination', models.CharField(default='PARTNER_BALANCE', max_length=255, verbose_name='settlement destination')),
                ('allowed_payment_method', models.CharField(default='INSTANT_FUNDING_SOURCE', max_length=255, verbose_name='allowed payment method')),
                ('description', models.CharField(max_length=255, verbose_name='description', blank=True)),
                ('note_to_payee', models.CharField(max_length=255, verbose_name='note to payee', blank=True)),
                ('custom', models.CharField(max_length=127, verbose_name='custom', blank=True)),
                ('invoice_number', models.CharField(max_length=127, verbose_name='invoice number')),
                ('purchase_order', models.CharField(max_length=127, verbose_name='purchase order')),
                ('soft_descriptor', models.CharField(max_length=22, verbose_name='soft descriptor')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='created at')),
                ('last_modified', models.DateTimeField(auto_now=True, verbose_name='last modified')),
                ('shipping_method', models.CharField(max_length=255, verbose_name='shipping method', blank=True)),
                ('shipping_phone_number', models.CharField(max_length=255, verbose_name='shipping phone number', blank=True)),
                ('shipping_address_line_1', models.CharField(max_length=100, verbose_name='shipping address line 1')),
                ('shipping_address_line_2', models.CharField(max_length=100, verbose_name='shipping address line 2', blank=True)),
                ('shipping_address_city', models.CharField(max_length=50, verbose_name='shipping address city')),
                ('shipping_address_country', models.CharField(max_length=2, verbose_name='shipping address country')),
                ('shipping_address_postal_code', models.CharField(max_length=20, verbose_name='shipping address postal code', blank=True)),
                ('shipping_address_state', models.CharField(max_length=100, verbose_name='shipping address state', blank=True)),
                ('shipping_address_phone', models.CharField(max_length=50, verbose_name='shipping address phone')),
                ('shipping_address_normalization_status', models.CharField(default='UNKNOWN', max_length=50, verbose_name='shipping address normalization status')),
                ('shipping_address_type', models.CharField(default='HOME_OR_WORK', max_length=50, verbose_name='shipping address type')),
                ('shipping_address_recipient_name', models.CharField(max_length=127, verbose_name='shipping address recipient name')),
                ('payment', models.ForeignKey(related_name='transactions', verbose_name='payment', to='django_paypal.PaypalPayment', on_delete=models.deletion.PROTECT)),
            ],
            options={
                'verbose_name': 'Paypal Transaction',
                'verbose_name_plural': 'Paypal Transactions',
            },
        ),
        migrations.AddField(
            model_name='paypalitem',
            name='transaction',
            field=models.ForeignKey(related_name='items', verbose_name='transaction', to='django_paypal.PaypalTransaction', on_delete=models.deletion.PROTECT),
        ),
    ]



from django.contrib import admin

from .models import PaypalOrder, PaypalAPIResponse


class PaypalOrderAdmin(admin.ModelAdmin):
    pass


admin.site.register(PaypalOrder, PaypalOrderAdmin)


class PaypalAPIResponseAdmin(admin.ModelAdmin):
    pass


admin.site.register(PaypalAPIResponse, PaypalAPIResponseAdmin)



from requests import RequestException


class PaypalAuthFailure(RequestException):
    pass


class PaypalAPIError(RequestException):
    pass


class PaypalWebhookVerificationError(BaseException):
    pass


class PaypalOrderAlreadyCapturedError(RequestException):
    pass
